function vants(niter,directions,positions,rulestring,graphlines,acolorbar,verbose,xmax,ymax)
% function vants(niter, directions, positions, rulestring, graphlines, colorbar, verbose, xmax, ymax)
%
% Implements an arbitrary number of generalized Langton virtual ants, or
% "vants".  The vants move over an infinite two-dimensional surface
% discretized into cells.  In this implementation, the available space in
% which the vants move is expanded as needed.  When multiple vants occupy
% the same cell in the same time step, their actions occur sequentially,
% with precedence being determined by their index in the directions
% prameter.  Thus, if directions = ['S' 'W'], then the vant initially
% facing South would perform its actions before the vant initially facing
% West if they were both on the same cell at the start of an iteration.
%
% The available parameters are as follows:
%
% niter is the number of steps the vant(s) take(s); it should be an integer.
%
% directions are the initial directions the vants face;
%     it should be a row vector consisting of elements taking the values
%     'N','E','S' or 'W'.  The length of the directions parameter
%     determines the number of vants to be simulated.
%
% positions is an m-by-2 matrix of initial coordinates for the vants, where
%     m = number of vants; The coordinate system begins in the top left
%     corner with [1,1] and ends at the bottom right corner with
%     [xmax,ymax], where xmax and ymax are the number of rows and columns,
%     respectively, to begin with in the vants' world.  For example, with
%     two vants, the positions matrix might be defined as:
%
%          [1 1; 2 2]
%
%     This would start the first vant at the top left cell, and the second
%     vant at the cell which is immediately SouthEast of that cell.  Note,
%     however, that the actual positions are unimportant; only the relative
%     positions of the vants matter.  As the vants move along after their
%     initial configuration, the two-dimensional space is expanded to
%     accomodate their movements in any direction anyway, so the starting
%     coordinates lose their meaning.
%
% rulestring should be a row vector consisting of 0's and 1's that
%     describe the actions that each vant takes on each step.  The number
%     of elements in the rulestring dictates how many possible states there
%     are for each cell.  Thus, if there are n elements in rulestring,
%     there are n possible states, labelled 0 through n-1.  The (k+1)th
%     element in the rulestring then represents the action to be performed
%     by the vant when it finds itself on a cell having state k.  In all
%     cases, the vant first changes the cell's state to k+1 modulo n.
%     Then, it has one of two choices: if rulestring(k+1) is a 0, then the
%     vant turns 90 degrees to the left, while if rulestring(k+1) is a 1
%     the vant turns 90 degrees to the right.  After making the appropriate
%     turn, the vant moves forward one cell and the next step begins.  All
%     vants share the same rulestring.
%
% graphlines determines the way the results will be plotted:
%      'lines' - graph lines will depict the separation between cells
%      'nolines' - graph lines will be omitted
%
% colorbar determines whether there will be a colorbar displayed:
%      'colorbar' - a colorbar will appear next to the plot
%      'nocolorbar' - no colorbar will be displayed
%
% xmax and ymax are arguments that can specify an absolute size in cells
%      for the cellular automaton simulation.
%
% verbose determines whether the function will output the positions and
%      directions of the ants.  If this information is desired, then
%      this argument should be set to 'verbose'.
%
%  Example usage: vants(3800,['E' 'S'],[1 1;1 17],[1 0],'lines','nocolorbar')
%
% (C) Dylan Hirsch-Shell, Nov 2006


xpos = positions(:,1);
ypos = positions(:,2);
cells = zeros(max(ypos)+1,max(xpos)+1);

if nargin>8
    cells = zeros(ymax+1,xmax+1);
end

nvants = length(directions);
nstates = length(rulestring);

% direction_vectors.N = [0 -1];
% direction_vectors.E = [1 0];
% direction_vectors.S = [0 1];
% direction_vectors.W = [-1 0];

for i = 1:niter
    if (min(xpos) <= 1)
        cells = [zeros(size(cells,1),1) cells];
        xpos = xpos + 1;
    end
    if (max(xpos) >= size(cells,2)-1)
        cells = [cells zeros(size(cells,1),1)];
    end
    if (min(ypos) <= 1)
        cells = [zeros(1,size(cells,2)); cells];
        ypos = ypos + 1;
    end
    if (max(ypos) >= size(cells,1)-1)
        cells = [cells; zeros(1,size(cells,2))];
    end
    for j = 1:nvants
        rule = rulestring(cells(ypos(j),xpos(j))+1);
        cells(ypos(j),xpos(j)) = mod(cells(ypos(j),xpos(j))+1,nstates);
        switch rule
            case 0
                if directions(j) == 'N'
                    directions(j) = 'W';
                    xpos(j) = xpos(j) - 1;
                elseif directions(j) == 'E'
                    directions(j) = 'N';
                    ypos(j) = ypos(j) - 1;
                elseif directions(j) == 'S'
                    directions(j) = 'E';
                    xpos(j) = xpos(j) + 1;
                elseif directions(j) == 'W'
                    directions(j) = 'S';
                    ypos(j) = ypos(j) + 1;
                end
            case 1
                if directions(j) == 'N'
                    directions(j) = 'E';
                    xpos(j) = xpos(j) + 1;
                elseif directions(j) == 'E'
                    directions(j) = 'S';
                    ypos(j) = ypos(j) + 1;
                elseif directions(j) == 'S'
                    directions(j) = 'W';
                    xpos(j) = xpos(j) - 1;
                elseif directions(j) == 'W'
                    directions(j) = 'N';
                    ypos(j) = ypos(j) - 1;
                end
        end
    end
end

figure
pcolor(cells)
set(gca,'YDir','reverse','XTick',[],'YTick',[])
axis image
axis([1 size(cells,2) 1 size(cells,1)]);
hold on
direction_arrows.N = '^';
direction_arrows.E = '>';
direction_arrows.S = 'v';
direction_arrows.W = '<';
linestylestring = {};
for vant = 1:nvants
    linestylestring{end+1} = direction_arrows.(directions(vant));
end
set(gca,'LineStyleOrder',linestylestring);
for vant = 1:nvants
    colororder = get(gca,'ColorOrder');
    plot(xpos(vant)+0.5,ypos(vant)+0.5,direction_arrows.(directions(vant)),'MarkerSize',10,'MarkerFaceColor',colororder(vant,:),'MarkerEdgeColor',colororder(vant,:));
end
hold off
map = 1-repmat((find(rulestring==rulestring)'-1)/(length(rulestring)-1),1,3);
caxis([0 nstates-1])
colormap(gca,map)
if nargin > 4
    if strcmp(graphlines,'nolines')
        shading flat
        axis off
    else
        shading faceted
    end
end
if nargin > 5
    if strcmp(acolorbar,'colorbar')
        colorbar('YTickMode','manual','YTick',...
            (find(rulestring==rulestring)-0.5)*(nstates-1)/nstates,'YTickLabel',...
            {find(rulestring==rulestring)-1},'TickLength',[0;0])
    end
end
if nargin > 6
    if strcmp(verbose,'verbose')
        disp(xpos); disp(ypos); disp(directions)
    end
end

